/*
 * _print_macros_xc.h
 *
 *  Created on: 12. mars 2021
 *      Author: teig
 */


#ifndef PRINT_MACROS_XC_H_
#define PRINT_MACROS_XC_H_

    #if (((defined DEBUG_PRINT_ENABLE_ALL) and (DEBUG_PRINT_ENABLE_ALL==1)) or /* from "makefile" */ \
         ((defined DEBUG_PRINT_ENABLE)     and (DEBUG_PRINT_ENABLE==1    )))

        #if (WARNINGS==1) // From "makefile"
            #warning Using /lib_logging (that is why sprintf is used when floats)
        #endif

        #include "debug_conf.h"
        #include "debug_print.h"

        // In makefile:
        //   USED_MODULES = .. lib_logging ..
        //   XCC_FLAGS += -DDEBUG_PRINT_ENABLE=1

        /*
        AN00239: https://www.xmos.ai/download/AN00239:-Using-the-logging-library(1.0.0rc1).pdf

        A limited functionality version of printf that is low memory.
        This function works like C-standard printf except that it only accepts d, x, s, u and c format
        specifiers with no conversions.
        The p format specifier is treated the same as a x.
        The capital version of each format specifier performs the same as the lower case equivalent.
        Any alignment or padding characters are simply ignored.
        The function uses the functions from print.h to do the underlying printing.
        Unlike printf this function has no return temp_degC.
        Whether the function does any output can be controlled via defines such as DEBUG_PRINT_ENABLE or
        DEBUG_PRINT_ENABLE_[debug unit name] in the applications debug_conf.h
        */

       #define debug_print(fmt, ...) do { if(DO_DEBUG_PRINT) debug_printf(fmt, __VA_ARGS__); } while (0) // In /lib_logging/api/debug_print.h

    #elif (DO_DEBUG_PRINT==1)
        #if (WARNINGS==1) // From "makefile"
            #warning jtag printing not as reliable
        #endif
        // See https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c
        #define debug_print(fmt, ...) do { if(DO_DEBUG_PRINT) printf(fmt, __VA_ARGS__); } while (0)
    #else
        #if (WARNINGS==1) // From "makefile"
            #warning No printing
        #endif
        #define debug_print(fmt, ...)
    #endif

    #ifndef DEBUG_PRINTF_BUFSIZE
        #define SPRINTF_LEN 300
    #else
        #define SPRINTF_LEN DEBUG_PRINTF_BUFSIZE // From "debug_conf.h", to here and /lib_logging
    #endif

    /*
     _____      _       _
     |  __ \    (_)     | |
     | |__) | __ _ _ __ | |_   _ __ ___   __ _  ___ _ __ ___  ___
     |  ___/ '__| | '_ \| __| | '_ ` _ \ / _` |/ __| '__/ _ \/ __|
     | |   | |  | | | | | |_  | | | | | | (_| | (__| | | (_) \__ \
     |_|   |_|  |_|_| |_|\__| |_| |_| |_|\__,_|\___|_|  \___/|___/
     TO AVOID THEM CLUTTERING THE CODE
    */

    #define CLIENT_ROOT_STR "R"
    #define CLIENT_STR      "C"
    #define SERVER_STR      "S"

    #if (DO_DEBUG_PRINT==0)
        #define PRINT_BANNER_ROOT(ctx,str1,str2,temp,str3)
    #else
        #define PRINT_BANNER_ROOT(ctx,str1,str2,temp,str3) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) [%.4f] \"%s\" as \"%s\" on %s %s\n",  \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      temp, str2, str3,\
                      __DATE__, __TIME__); \
                debug_printf ("%s", chars_str); \
            } while (0)
    #endif

    #if (DO_DEBUG_PRINT==0)
        #define PRINT_BANNER(ctx,str1,str2,temp)
    #else
        #define PRINT_BANNER(ctx,str1,str2,temp) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) [%.4f] \"%s\"\n",  \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      temp, str2); \
                debug_printf ("%s", chars_str); \
            } while (0)
    #endif


    #if ((DO_PRINT_VALUES==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_VALUES_WITH_TIME(ctx,str1,tmr,temp)
        #define PRINT_VALUES_WITH_TIME_R(ctx,str1,tmr,temp,root)
        #define PRINT_VALUES_WITH_TIME_MOD(ctx,str1,tmr,temp,mod)
        #define print_values_with_time_mod(ctx,str1,tmr,temp,mod)
    #else
        #define PRINT_VALUES_WITH_TIME(ctx,str1,tmr,temp) \
            do { \
                time32_t time_now_ticks; \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                tmr :> time_now_ticks; \
                len = sprintf (chars_str,     \
                      "\n(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f] at %u ms\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      temp_degC, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2], \
                      time_now_ticks/XS1_TIMER_KHZ); \
                debug_printf ("%s", chars_str); \
            } while(0)

        #define PRINT_VALUES_WITH_TIME_MOD(ctx,str1,tmr,temp,mod) \
            do { \
                if ((ctx.cycle_cnt%mod) == 0) { \
                    time32_t time_now_ticks; \
                    int  len; \
                    char chars_str [SPRINTF_LEN]; \
                    tmr :> time_now_ticks; \
                    len = sprintf (chars_str,     \
                          "\n(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f] at %u ms\n", \
                          ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                          temp, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2], \
                          time_now_ticks/XS1_TIMER_KHZ); \
                    debug_printf ("%s", chars_str); \
                }  \
            } while(0)

        void print_values_with_time_mod (node_context_t ctx, char str1[], timer tmr, const temp_degC_t temp, const unsigned mod) {
            PRINT_VALUES_WITH_TIME_MOD (ctx, str1, tmr, temp, mod);
        }

    #endif

    #if ((DO_PRINT_VALUES==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_VALUES(ctx,str1,temp)
        #define print_values(ctx,str1,temp)
        #define PRINT_NL_VALUES(ctx,str1,temp)
        #define PRINT_VALUES_MOD(ctx,str1,temp,mod)
        #define print_values_mod(ctx,str1,temp,mod)
        #define PRINT_NL_VALUES_MOD(ctx,str1,temp,mod)
    #else
        #define PRINT_VALUES(ctx,str1,temp) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      temp, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2]); \
                debug_printf ("%s", chars_str); \
            } while(0)

        void print_values (node_context_t ctx, char str1[], const temp_degC_t temp) {
            PRINT_VALUES(ctx,str1,temp);
        }

        #define PRINT_NL_VALUES(ctx,str1,temp) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "\n(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      temp, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2]); \
                debug_printf ("%s", chars_str); \
            } while(0)

        #define PRINT_VALUES_MOD(ctx,str1,temp,mod) \
            do { \
                if ((ctx.cycle_cnt%mod) == 0) { \
                    int  len; \
                    char chars_str [SPRINTF_LEN]; \
                    len = sprintf (chars_str,     \
                          "(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f]\n", \
                          ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                          temp, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2]); \
                    debug_printf ("%s", chars_str); \
                }  \
            } while(0)

        void print_values_mod (node_context_t ctx, char str1[], const temp_degC_t temp, const unsigned mod) {
            PRINT_VALUES_MOD (ctx, str1, temp, mod);
        }

        #define PRINT_NL_VALUES_MOD(ctx,str1,temp,mod) \
            do { \
                if ((ctx.cycle_cnt%mod) == 0) { \
                    int  len; \
                    char chars_str [SPRINTF_LEN]; \
                    len = sprintf (chars_str,     \
                          "\n(%u %u):%s(%u) [%.8f] from [%.4f][%.4f][%.4f]\n", \
                          ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                          temp, ctx.temps_degC[0], ctx.temps_degC[1], ctx.temps_degC[2]); \
                    debug_printf ("%s", chars_str); \
                }  \
            } while(0)
    #endif


    #if ((DO_PRINT_JOULE==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_JOULE_MOD(ctx,str1,joule,mod)
    #else

        #define PRINT_JOULE_MOD(ctx,str1,joule,mod) \
            do { \
                if ((ctx.cycle_cnt%mod) == 0) { \
                    int  len; \
                    char chars_str [SPRINTF_LEN]; \
                    len = sprintf (chars_str,     \
                          "(%u %u):%s(%u) Joule:[%.4f] from heatcap_vals:[%.4f][%.4f][%.4f]\n", \
                          ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                          joule, ctx.heatcap_vals[0], ctx.heatcap_vals[1], ctx.heatcap_vals[2]); \
                    debug_printf ("%s", chars_str); \
                }  \
            } while(0)
    #endif


    #if ((DO_PRINT_PROGESS==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_PROGESS(ctx,str1,val,val2)
        #define PRINT_NL_PROGESS(ctx,str1,val,val2)
    #else
        #define PRINT_PROGESS(ctx,str1,val1,val2) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "P(%u %u):%s(%u) %u %u\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      val1, val2); \
                debug_printf ("%s", chars_str); \
            } while(0)

        #define PRINT_NL_PROGESS(ctx,str1,val1,val2) \
            do { \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "\nP(%u %u):%s(%u) %u %u\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      val1, val2); \
                debug_printf ("%s", chars_str); \
            } while(0)
    #endif


    #if ((DO_PRINT_INIT_WT==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_INIT_WT0
        #define PRINT_INIT_WT1_ODD
        #define PRINT_INIT_WT1_EVEN
        #define PRINT_INIT_WT2
        #define PRINT_INIT_WT_SUMMARY
    #elif (DO_PRINT_INIT_WT==1)
        #define PRINT_INIT_WT0 do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,"(%u %u):wt[0] = dpt=%.4f * condu_vals=%.4f / heatcap_vals=%.4f\n", \
                    iof_row, iof_col, \
                    params.dpt_val,  \
                    params.condu_vals[iof_row][(iof_col+INIT_NUM_COLS-1) % INIT_NUM_COLS],  \
                params.heatcap_vals[iof_row][iof_col]); \
            debug_printf ("%s", chars_str); \
        } while(0)

        #define PRINT_INIT_WT1_ODD do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,"(%u %u):wt[1-odd] = dpt=%.4f * condu_vals=%.4f / heatcap_vals=%.4f\n", \
                    iof_row, iof_col, \
                    params.dpt_val, \
                    params.conduvert_vals[iof_row][iof_col>>1], \
                    params.heatcap_vals[iof_row][iof_col]); \
                debug_printf ("%s", chars_str); \
            } while(0)


        #define PRINT_INIT_WT1_EVEN do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,"(%u %u):wt[1-even] = dpt=%.4f * condu_vals=%.4f / heatcap_vals=%.4f\n", \
                    iof_row, iof_col, \
                    params.dpt_val,  \
                    params.conduvert_vals[(iof_row+INIT_NUM_ROWS-1) % INIT_NUM_ROWS][iof_col>>1], \
                    params.heatcap_vals[iof_row][iof_col]); \
                debug_printf ("%s", chars_str); \
            } while(0)

        #define PRINT_INIT_WT2 do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,"(%u %u):wt[2] = dpt=%.4f * condu_vals=%.4f / heatcap_vals=%.4f\n", \
                    iof_row, iof_col, \
                    params.dpt_val,  \
                    params.condu_vals[iof_row][iof_col],  \
                    params.heatcap_vals[iof_row][iof_col]); \
                debug_print ("%s", chars_str);\
            } while(0)

        #define PRINT_INIT_WT_SUMMARY do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
            len = sprintf (chars_str, "(%u %u):wt %.4f %.4f %.4f\n", iof_row, iof_col, wt[0], wt[1], wt[2]); \
            debug_printf ("%s", chars_str); \
        } while(0)
    #endif


    #if ((DO_PRINT_INIT_TEMP==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_INIT_TEMP
    #elif (DO_PRINT_INIT_TEMP==1)
        #define PRINT_INIT_TEMP do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
            len = sprintf (chars_str, "(%u %u):temp %.4f\n", iof_row, iof_col, return_temp); \
            debug_printf ("%s", chars_str); \
        } while(0)
    #endif


    #if ((DO_PRINT_CALC_TEMP==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_CALC_TEMP
    #elif (DO_PRINT_CALC_TEMP==1)
        #define PRINT_CALC_TEMP do { \
            int  len; \
            char chars_str [SPRINTF_LEN]; \
            len = sprintf (chars_str, "                (%u %u):t(%u) r:%.4f = o:%.4f + t1:%.4f + t2:%.4f + t3:%.4f ", \
                    context.iof_row, context.iof_col, context.cycle_cnt, \
                    temp_return, tempold, term1, term2, term3); \
            for (unsigned index = 0; index < NUM_CONNS_PER_NODE; index++) { \
                len += sprintf (len+chars_str, "[wt:%.4f t:%.4f] ", context.wt[index], context.temps_degC[index]); \
            } \
            len += sprintf (len+chars_str, "%s", "\n"); \
            debug_printf ("%s", chars_str); \
        } while(0)
    #endif

    #if ((DO_PRINT_SYNCH==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_SYNCH(ctx,str1,val1,val2,val3,val4,val5)
        #define print_synch(ctx,str1,val1,val2,val3,val4,val5)
        #define PRINT_SYNXH(ctx,str1,val1,val2,val3)
        #define print_synxh(ctx,str1,val1,val2,val3)
    #elif (DO_PRINT_SYNCH==1)

        #define PRINT_SYNCH(ctx,str1,val1,val2,val3,val4,val5) \
            do { \
                time32_t time_now_ticks; \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "  (%u %u):%s(%u) i:%u SYNcH:[cli-calls:%u synch-ret:%u] key[%u] key_for_client:[%u]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      val1,val2,val3,val4,val5); \
                debug_printf ("%s", chars_str); \
            } while(0)

        void print_synch (node_context_t ctx, char str1[], const unsigned val1, const unsigned val2, const unsigned val3, const unsigned val4, const unsigned val5) {
            PRINT_SYNCH(ctx,str1,val1,val2,val3,val4,val5);
        }

        #define PRINT_SYNXH(ctx,str1,val1,val2,val3) \
            do { \
                time32_t time_now_ticks; \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) SYNxH:[%u-%u-%u]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, \
                      val1,val2,val3); \
                debug_printf ("%s", chars_str); \
            } while(0)

        void print_synxh (node_context_t ctx, char str1[], const unsigned val1, const unsigned val2, const unsigned val3) {
            PRINT_SYNXH(ctx,str1,val1,val2,val3);
        }

    #endif

    #if ((DO_PRINT_RX_TX==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_TX(ctx,str1,index,tx)
        #define print_tx(ctx,str1,index,tx)
        #define PRINT_RX(ctx,str1,index,rx)
        #define print_rx(ctx,str1,index,rx)
    #elif (DO_PRINT_RX_TX==1)

        // Combined RX_TX or TX_RX is not a good idea since values then will appear out of sequence in the log

        #define PRINT_TX(ctx,str1,index,tx) \
            do { \
                time32_t time_now_ticks; \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) i=%u TX:[%.4f]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, index, tx); \
                debug_printf ("%s", chars_str); \
            } while(0)

        void print_tx (node_context_t ctx, char str1[], const unsigned index, const temp_degC_t tx) {
            PRINT_TX(ctx,str1,index,tx);
        }

        #define PRINT_RX(ctx,str1,index,rx) \
            do { \
                time32_t time_now_ticks; \
                int  len; \
                char chars_str [SPRINTF_LEN]; \
                len = sprintf (chars_str,     \
                      "(%u %u):%s(%u) i=%u RX:[%.4f]\n", \
                      ctx.iof_row, ctx.iof_col, str1, ctx.cycle_cnt, index, rx); \
                debug_printf ("%s", chars_str); \
            } while(0)

        void print_rx (node_context_t ctx, char str1[], const unsigned index, const temp_degC_t rx) {
            PRINT_RX(ctx,str1,index,rx);
        }

    #endif

    #if ((DO_PRINT_FIX_SYNCH==0) or (DO_DEBUG_PRINT==0))
        #define PRINT_FIX_SYNCH(ctx,keycli,keyserv,iofcli,cnt)
        #define print_fix_synch(ctx,keycli,keyserv,iofcli,cnt)
    #elif (DO_PRINT_FIX_SYNCH==1)

        #define PRINT_FIX_SYNCH(ctx,keycli,keyserv,iofcli,cnt) do { \
            debug_print ("FIX (%u %u) keycli:%u keyserv:%u iofcli(%u) cnt(%u)\n", ctx.iof_row, ctx.iof_col, keycli, keyserv, iofcli, cnt); } while(0)

        void print_fix_synch (node_context_t ctx, const unsigned keycli, const unsigned keyserv, const unsigned iofcli, const unsigned cnt) {
            PRINT_FIX_SYNCH (ctx,keycli,keyserv,iofcli,cnt);
        }
    #endif

#endif /* PRINT_MACROS_XC_H_ */
